/*
 * Copyright (c) 2022-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * Copyright (c) 2020 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

/*
 * Abstract:
 *    OSM Key Manager for key configuration of multiple classes
 *
 * Author:
 *    Or Nechemia, Mellanox
 */

#ifndef OSM_KEY_MGR_H
#define OSM_KEY_MGR_H

#include <iba/ib_types.h>
#include <complib/cl_types_osd.h>
#include <complib/cl_dispatcher.h>
#include <opensm/osm_subnet.h>
#include <opensm/osm_log.h>
#include <opensm/osm_vendor_specific.h>
#include <opensm/osm_congestion_control.h>
#include <opensm/osm_n2n.h>

#define OSM_KEY_TYPE_MANAGER		0
#define OSM_KEY_TYPE_NEIGHBOR_SW	1

struct osm_opensm;

/****s* OpenSM: KeyManager/osm_key_mgr_t
* NAME
*	osm_key_mgr_t
*
* DESCRIPTION
*	OpenSM Key manager object structure.
*	This structure should be treated as opaque and should be manipulated
*	only through the provided functions.
*
* SYNOPSIS
*/
typedef struct osm_key_mgr {
	struct osm_opensm * p_osm;
	const osm_subn_opt_t * p_opt;
	uint8_t key_seed[64];
} osm_key_mgr_t;
/*
* FIELDS
*	p_osm
*		Pointer to OpenSM object
*
*	p_opt
*		Pointer to SM options structure
*
*	key_seed
*		Seed used for calculating key per port.
*********/

/****f* OpenSM: KeyManager/osm_key_mgr_init
* NAME
* 	osm_key_mgr_init
*
* DESCRIPTION
*	Initializes osm_key_mgr_t structure for use
*
* SYNOPSIS
*/
int osm_key_mgr_init(osm_key_mgr_t * p_key_mgr, struct osm_opensm * p_osm);
/*
* PARAMETERS
*	p_key_mgr
*
*		The key manager instance to initalize.
*	p_osm
*		Pointer to OpenSM object
*
* RETURN VALUE
*	Zero on success, nonzero otherwise.
*
* NOTES
*	osm_key_mgr_destroy must be called to free resources used by the key manager.
*
* SEE ALSO
*	osm_key_mgr_destroy
*********/

/****f* OpenSM: KeyManager/osm_key_mgr_destroy
* NAME
* 	osm_key_mgr_destroy
*
* DESCRIPTION
*	Frees resources used by the key manager
*
* SYNOPSIS
*/
void osm_key_mgr_destroy(osm_key_mgr_t * p_key_mgr);
/*
* PARAMETERS
*	p_key_mgr
*		The key manager instance to destroy.
*
* RETURN VALUE
*	None
*
* NOTES
*	osm_key_mgr_destroy must be called to free resources used by the key manager.
*
* SEE ALSO
*	osm_key_mgr_init
*********/

/****f* OpenSM: KeyManager/osm_key_mgr_db_is_initialized
* NAME
* 	osm_key_mgr_db_is_initialized
*
* DESCRIPTION
*	Checks whether the database was initialized.
*
* SYNOPSIS
*/
boolean_t osm_key_mgr_db_is_initialized(osm_opensm_t * p_osm, uint8_t mgmt_class, uint8_t key_type);
/*
* PARAMETERS
*	p_osm
*		Pointer to an osm_opensm_t object.
*
*	mgmt_class
*		Management class. Currently supported: CC, VS, N2N
*
*	key_type
*		Boolean indicates which N2N Class key is referred -
*		manager key or node to node key.
*
* RETURN VALUE
*	Boolean indicating whether the db was initialized.
*
* SEE ALSO
*********/

/****f* OpenSM: KeyManager/osm_key_mgr_generate_key
* NAME
* 	osm_key_mgr_generate_key
*
* DESCRIPTION
* 	Get key for port by its guid
*
* SYNOPSIS
*/
ib_net64_t osm_key_mgr_generate_key(struct osm_opensm * p_osm, osm_port_t * p_port,
				    uint8_t mgmt_class, uint8_t key_type);
/*
* PARAMETERS
*	p_osm
*		Pointer to an osm_opensm_t object.
*
*	p_port
*		Pointer to port
*
*	mgmt_class
*		Management class. Currently supported: CC, VS, N2N
*
*	neighbor_key
*		Boolean indicates of which N2N Class key is referred -
*		manager key or node to node key.
*
* RETURN VALUE
*	Class key for input port.
*
* SEE ALSO
*
*********/

void osm_key_mgr_get_cached_key(struct osm_opensm * p_osm, osm_port_t * p_port,
				uint8_t mgmt_class, uint8_t key_type);

void osm_key_mgr_set_cached_key(struct osm_opensm * p_osm, osm_port_t * p_port,
				uint8_t mgmt_class, uint8_t key_type);

#endif				/* ifndef OSM_KEY_MGR_H */
